<?php

/**
 * @package plugin LogBadPasswords
 * @copyright (C) 2010-2012 RicheyWeb - www.richeyweb.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 * LogBadPasswords Copyright (c) 2012 Michael Richey.
 * LogBadPasswords is licensed under the http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 *
 * LogBadPasswords version 1.2 for Joomla 1.6.x devloped by RicheyWeb
 *
 */
// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

/**
 * LogBadPasswords system plugin
 */
class plgUserLogBadPasswords extends JPlugin {

    /**
     * Constructor
     *
     * @access	protected
     * @param	object	$subject The object to observe
     * @param 	array   $config  An array that holds the plugin configuration
     * @since	1.0
     */
    function plgUserLogBadPasswords(&$subject, $config) {
        parent::__construct($subject, $config);
        $this->loadLanguage();
    }

    /**
     * @param	string	$context	The context for the data
     * @param	int		$data		The user id
     * @param	object
     *
     * @return	boolean
     * @since	1.6
     */
    function onContentPrepareData($context, $data) {

        $app = JFactory::getApplication();
        if ($app->isSite())
            return true;

        // Check we are manipulating a valid form.
        if (!in_array($context, array('com_users.profile', 'com_users.user', 'com_users.registration', 'com_admin.profile'))) {
            return true;
        }
        if (is_object($data)) {
            $userId = isset($data->id) ? $data->id : 0;

            if (!isset($data->logbadpasswords) and $userId > 0) {
                // Load the profile data from the database.
                $db = JFactory::getDbo();
                $query = $db->getQuery(true);
                $query->select('profile_value')->from('#__user_profiles');
                $query->where('profile_key = ' . $db->quote('logbadpasswords.passwords') . ' AND user_id = ' . $userId);
                $db->setQuery($query);
                $badpasswords = $db->loadResult();

                // Check for a database error.
                if ($db->getErrorNum()) {
                    $this->_subject->setError($db->getErrorMsg());
                    return false;
                }

                // Merge the profile data.

                JForm::addFormPath(dirname(__FILE__) . '/profiles');
                $form = JForm::getInstance('plg_user_logbadpasswords.form', 'logbadpasswords');

                $data->logbadpasswords = array('badpasswords' => $badpasswords);
            }
        }
        return true;
    }

    /**
     * @param	JForm	$form	The form to be altered.
     * @param	array	$data	The associated data for the form.
     *
     * @return	boolean
     * @since	1.6
     */
    function onContentPrepareForm($form, $data) {
        $app = JFactory::getApplication();
        if ($app->isSite())
            return true;

        if (!($form instanceof JForm)) {
            $this->_subject->setError('JERROR_NOT_A_FORM');
            return false;
        }

        // Check we are manipulating a valid form.
        $name = $form->getName();
        if (!in_array($name, array('com_admin.profile', 'com_users.user', 'com_users.profile', 'com_users.registration'))) {
            return true;
        }

        // Add the registration fields to the form.
        JForm::addFormPath(dirname(__FILE__) . '/profiles');
        $form->loadFile('logbadpasswords', false);
    }

    function onUserLogin($user, $options) {
        $app = JFactory::getApplication();
        // some basic tests
        if ($app->isAdmin()) {
            if (!$this->params->get('runinadmin', false)) {
                return true;
            }
        }else {
            if (!$this->params->get('runinsite', false)) {
                return true;
            }
        }        

        // if there aren't any bad passwords to store, then we exit
        $newbadpasswords = $app->getUserState('plg_logbadpasswords', array());
        if (!count($newbadpasswords))
            return true;

        // now we need a db object
        $db = JFactory::getDbo();

        // we need the user id to continue
        $query = $db->getQuery(true);
        $query->select('id')->from('#__users')->where('username = ' . $db->quote($user['username']));
        $db->setQuery($query);
        $user['id'] = $db->loadResult();
        
        // now we verify that we're supposed to store the password for this user
        $juser = JFactory::getUser($user['id']);
        foreach($juser->groups as $group) {
            if(in_array($group,$this->params->get('badgroups',array()))) return true;
        }

        // now we check to see if there were any previously stored bad passwords
        $query = $db->getQuery(true);
        $query->select(array('profile_value','user_id'))->from('#__user_profiles')->where('profile_key = "logbadpasswords.passwords" AND user_id = ' . $user['id']);
        $db->setQuery($query);
        $oldbadpasswords = $db->loadObjectList();
        if (count($oldbadpasswords)) {
            $query = $db->getQuery(true);
            $oldbadpasswords = explode(', ', trim($oldbadpasswords->profile_value));
            foreach ($oldbadpasswords as $bp)
                if(strlen(trim($bp))) $newbadpasswords[] = $bp;
            asort($newbadpasswords);
            $badpasswords = implode(', ', $this->_removeValid($user['password'],$newbadpasswords));
            $query->update('#__user_profiles')->set('profile_value = ' . $db->quote($badpasswords))->where('profile_key = "logbadpasswords.passwords" AND user_id = ' . $user['id']);
        } else {
            $query = $db->getQuery(true);
            asort($newbadpasswords);
            $badpasswords = implode(', ', $this->_removeValid($user['password'],$newbadpasswords));
            $query->insert('#__user_profiles (user_id,profile_key,profile_value)')->values($user['id'] . ',' . $db->quote('logbadpasswords.passwords') . ',' . $db->quote($badpasswords));
        }
        $db->setQuery($query);
        $db->query();
        return true;
    }

    function onUserAfterSave($data, $isNew, $result, $error) {
        // you can only modify this from admin
        $app = JFactory::getApplication();
        if ($app->isSite())
            return true;

        $userId = JArrayHelper::getValue($data, 'id', 0, 'int');

        if ($userId && $result && isset($data['logbadpasswords']) && (count($data['logbadpasswords']))) {
            try {
                $db = JFactory::getDbo();
                $db->setQuery(
                        'DELETE FROM #__user_profiles WHERE user_id = ' . $userId .
                        " AND profile_key = 'logbadpasswords.passwords'"
                );

                if (!$db->query()) {
                    throw new Exception($db->getErrorMsg());
                }

                $tuples = array();
                $order = 1;

                JForm::addFormPath(dirname(__FILE__) . '/profiles');
                $form = JForm::getInstance('plg_user_logpadpasswords.form', 'logbadpasswords');
                $values= '(' . $userId . ', ' . $db->quote('logbadpasswords.passwords') . ', ' . $db->quote($data['logbadpasswords']['badpasswords']) . ', ' . $order++ . ')';

                $db->setQuery('INSERT INTO #__user_profiles VALUES ' . $values);

                if (!$db->query()) {
                    throw new Exception($db->getErrorMsg());
                }
            } catch (JException $e) {
                $this->_subject->setError($e->getMessage());
                return false;
            }
        }

        return true;
    }

    /**
     * Remove all user profile information for the given user ID
     *
     * Method is called after user data is deleted from the database
     *
     * @param	array		$user		Holds the user data
     * @param	boolean		$success	True if user was succesfully stored in the database
     * @param	string		$msg		Message
     */
    function onUserAfterDelete($user, $success, $msg) {
        if (!$success) {
            return false;
        }

        $userId = JArrayHelper::getValue($user, 'id', 0, 'int');

        if ($userId) {
            try {
                $db = JFactory::getDbo();
                $db->setQuery(
                        'DELETE FROM #__user_profiles WHERE user_id = ' . $userId .
                        " AND profile_key = 'logbadpasswords.passwords'"
                );

                if (!$db->query()) {
                    throw new Exception($db->getErrorMsg());
                }
            } catch (JException $e) {
                $this->_subject->setError($e->getMessage());
                return false;
            }
        }

        return true;
    }

    function _removeValid($validpassword, $badpasswords) {
        if (in_array($validpassword, $badpasswords)) {
            unset($badpasswords[array_search($validpassword, $badpasswords)]);
        }
        return $badpasswords;
    }

}
